#!/usr/bin/env bash
###
 # Initializes general Python setup for development.
 ##


# Abort on error
set -e


# Import utility script.
source $(dirname $0)/utils.sh



###
 # Program main.
 ##
function main () {
    # Make sure we are not root.
    check_is_not_user "root"

    echo ""
    echo -e "${text_blue}Setting up Python for development.${text_reset}"

    # Check that deadsnakes Python ppa is present.
    search_string="deadsnakes-ubuntu-ppa-focal.list:deb"
    apt_repository_set="$(grep ^ /etc/apt/sources.list /etc/apt/sources.list.d/*)"
    if [[ ! "${apt_repository_set}" =~ "${search_string}" ]]
    then
        # Ppa not found. Add.
        sudo add-apt-repository ppa:deadsnakes/ppa -y > /dev/null 2>&1
    fi

    # Install all remotely recent Python versions.
    sudo apt-get install python3-pip -y > /dev/null 2>&1
    sudo apt-get install python3.7 python3.7-dev python3.7-venv -y > /dev/null 2>&1
    sudo apt-get install python3.8 python3.8-dev python3.8-venv -y > /dev/null 2>&1
    sudo apt-get install python3.9 python3.9-dev python3.9-venv -y > /dev/null 2>&1
    sudo apt-get install python3.10 python3.10-dev python3.10-venv -y > /dev/null 2>&1

    # Setup PipEnv.
    # Check that .bashrc file exists.
    username=$USER
    bashrc_file="/home/${username}/.bashrc"
    if [[ -f "${bashrc_file}" ]]
    then
        # Add PipEnv settings to .bashrc file.
        path_search_string="PATH=\"\$PATH:/home/${username}/.local/bin\""
        venv_in_project_search_string="PIPENV_VENV_IN_PROJECT=true"
        bashrc_contents="$(cat ${bashrc_file})"

        # Check if either value is not present.
        if [[ ! "${bashrc_contents}" =~ "${path_search_string}"
           || ! "${bashrc_contents}" =~ "${venv_in_project_search_string}" ]]
       then
            # Append.
            echo "" >> "${bashrc_file}"
            echo "" >> "${bashrc_file}"
            echo "# PipEnv Settings." >> "${bashrc_file}"
            if [[ ! "${bashrc_contents}" =~ "${path_search_string}" ]]
            then
                echo "${path_search_string}" >> "${bashrc_file}"
            fi
            if [[ ! "${bashrc_contents}" =~ "${venv_in_project_search_string}" ]]
            then
                echo "${venv_in_project_search_string}" >> "${bashrc_file}"
            fi
       fi
    fi

    # Install PipEnv.
    python3 -m pip install --user pipenv > /dev/null

    # Setup Poetry (PipEnv alternative).
    # Install Poetry.
    curl -sSL https://install.python-poetry.org | python3 -

    # Check that .bashrc file exists.
    if [[ -f "${bashrc_file}" ]]
    then
        # Add PipEnv settings to .bashrc file.
        path_search_string="PATH=\"\$PATH:/home/${username}/.local/bin\""
        bashrc_contents="$(cat ${bashrc_file})"

        # Check if either value is not present.
        if [[ ! "${bashrc_contents}" =~ "${path_search_string}" ]]
        then
            # Append.
            echo "" >> "${bashrc_file}"
            echo "" >> "${bashrc_file}"
            echo "# Poetry Settings." >> "${bashrc_file}"
            echo "${path_search_string}" >> "${bashrc_file}"

        fi
    fi

    # Add updated cd function to .bashrc file.
    # On cd into folders, this will load any Python environments in the direct folder that match ".env" or ".venv".
    # On cd out of folders, this will unload Python environments.
    if [[ -f "${bashrc_file}" ]]
    then
        cd_search_string="cd_venv_activate() {"
        bashrc_contents="$(cat ${bashrc_file})"

        script_url="https://git.brandon-rodriguez.com/python/example_projects/virtual_environments/-/raw/master/helper_script.sh"

        if [[ ! "${bashrc_contents}" =~ "${cd_search_string}" ]]
        then
            # Download script.
            wget -O "python_updated_cd_script.sh" "${script_url}" > /dev/null 2>&1

            # Add contents to .bashrc file.
            echo "" >> "${bashrc_file}"
            echo "" >> "${bashrc_file}"
            echo "" >> "${bashrc_file}"
            echo "$(cat ./python_updated_cd_script.sh)" >> "${bashrc_file}"
            echo ""

            # Remove downloaded script. It's no longer needed.
            rm -r "./python_updated_cd_script.sh"

        fi
    fi

    echo ""
    echo -e "${text_blue}Python setup complete.${text_reset}"
    echo ""
}


main
